==> ssh-2.2.0.tar.gz: ssh-2.2.0/lib/sshutil/sshnet/sshinetencode.c <==
/*
  
  sshinetencode.c       

  Author:
        Santeri Paavolainen <santtu@ssh.fi>

  Copyright:
        Copyright (c) 1999 SSH Communications Security, Finland
        All rights reserved

*/

#include "sshincludes.h"
#include "sshencode.h"
#include "sshinetencode.h"

size_t ssh_encode_ipaddr_array (unsigned char * buf, size_t bufsize,
                                const SshIpAddr ip)
{
  size_t got;

  if ((got = ssh_encode_array(buf, bufsize,
                              SSH_FORMAT_CHAR, ip->type,
                              SSH_FORMAT_UINT32, ip->mask_len,
                              SSH_FORMAT_DATA, ip->addr_data, SSH_IP_ADDR_LEN(ip),
                              SSH_FORMAT_END)) !=
      (1 + 4 + SSH_IP_ADDR_LEN(ip)))
    return 0;

  return got;
}

size_t ssh_encode_ipaddr_array_alloc (unsigned char ** buf_return,
                                      const SshIpAddr ip)
{
  size_t req, got;

  req = 1 + 4 + SSH_IP_ADDR_LEN(ip);

  if (buf_return == NULL)
    return req;

#ifdef _KERNEL
  if (!(*buf_return = ssh_kmalloc(req)))
    return 0;
#else
  *buf_return = ssh_xmalloc(req);
#endif /* _KERNEL */

  if ((got = ssh_encode_array(*buf_return, req,
                              SSH_FORMAT_CHAR, ip->type,
                              SSH_FORMAT_UINT32, ip->mask_len,
                              SSH_FORMAT_DATA, ip->addr_data, SSH_IP_ADDR_LEN(ip),
                              SSH_FORMAT_END)) != req) {
#ifdef _KERNEL
    ssh_kfree(*buf_return);
#else
    ssh_xfree(*buf_return);
#endif /* _KERNEL */
    *buf_return = NULL;
    return 0;
  }

  return got;
}

size_t ssh_decode_ipaddr_array (const unsigned char * buf,
                                size_t len,
                                SshIpAddr ip)
{
  size_t        point, got;

  point = 0;

  if ((got = ssh_decode_array(buf + point, len - point,
                              SSH_FORMAT_CHAR, &ip->type,
                              SSH_FORMAT_UINT32, &ip->mask_len,
                              SSH_FORMAT_END)) != 5)
      return 0;

  point += got;

  if ((got = ssh_decode_array(buf + point, len - point,
                              SSH_FORMAT_DATA, ip->addr_data, SSH_IP_ADDR_LEN(ip),
                              SSH_FORMAT_END)) != SSH_IP_ADDR_LEN(ip))
      return 0;

  point += got;

  /* Sanity check */
  if (!SSH_IP_IS4(ip) && !SSH_IP_IS6(ip))
    return 0;

  return point;
}

#ifndef _KERNEL
size_t ssh_encode_ipaddr_buffer (SshBuffer buffer, const SshIpAddr ip)
{
  size_t got;
  unsigned char tmpbuf[1 + 4 + 16];

  if ((got = ssh_encode_ipaddr_array(tmpbuf, sizeof(tmpbuf), ip)) == 0)
      return FALSE;

  ssh_buffer_append(buffer, tmpbuf, got);

  return TRUE;
}

size_t ssh_ipaddr_decode_buffer (SshBuffer buffer, SshIpAddr ip)
{
  size_t got;

  if ((got = ssh_decode_ipaddr_array(ssh_buffer_ptr(buffer), 
                                     ssh_buffer_len(buffer),
                                     ip)) == 0)
    return 0;

  ssh_buffer_consume(buffer, got);

  return got;
}
#endif

==> ssh-2.4.0.tar.gz: ssh-2.4.0/lib/sshutil/sshnet/sshinetencode.c <==
/*

  sshinetencode.c

  Author:
        Santeri Paavolainen <santtu@ssh.fi>

  Copyright:
        Copyright (c) 1999 SSH Communications Security, Finland
        All rights reserved

*/

#include "sshincludes.h"
#include "sshencode.h"
#include "sshinetencode.h"

size_t ssh_encode_ipaddr_array (unsigned char * buf, size_t bufsize,
                                const SshIpAddr ip)
{
  size_t got;

  if (ip->type == SSH_IP_TYPE_NONE)
    got = ssh_encode_array(buf, bufsize,
                           SSH_FORMAT_CHAR, ip->type,
                           SSH_FORMAT_END);
  else
    if ((got = ssh_encode_array(buf, bufsize,
                                SSH_FORMAT_CHAR, ip->type,
                                SSH_FORMAT_UINT32, ip->mask_len,
                                SSH_FORMAT_DATA,
                                ip->addr_data, SSH_IP_ADDR_LEN(ip),
                                SSH_FORMAT_END)) !=
        (1 + 4 + SSH_IP_ADDR_LEN(ip)))
      return 0;

  return got;
}

size_t ssh_encode_ipaddr_array_alloc (unsigned char ** buf_return,
                                      const SshIpAddr ip)
{
  size_t req, got;

  if (ip->type == SSH_IP_TYPE_NONE)
    req = 1;
  else
    req = 1 + 4 + SSH_IP_ADDR_LEN(ip);

  if (buf_return == NULL)
    return req;

#ifdef _KERNEL
  if (!(*buf_return = ssh_kmalloc(req)))
    return 0;
#else
  *buf_return = ssh_xmalloc(req);
#endif /* _KERNEL */

  got = ssh_encode_ipaddr_array(*buf_return, req, ip);

  if (got != req)
    {
#ifdef _KERNEL
      ssh_kfree(*buf_return);
#else
      ssh_xfree(*buf_return);
#endif /* _KERNEL */
      *buf_return = NULL;
      return 0;
    }

  return got;
}

size_t ssh_decode_ipaddr_array (const unsigned char * buf,
                                size_t len,
                                SshIpAddr ip)
{
  size_t point, got;

  point = 0;

  if ((got = ssh_decode_array(buf + point, len - point,
                              SSH_FORMAT_CHAR, &ip->type,
                              SSH_FORMAT_END)) != 1)
      return 0;

  point += got;

  if (ip->type == SSH_IP_TYPE_NONE)
    return point;

  if ((got = ssh_decode_array(buf + point, len - point,
                              SSH_FORMAT_UINT32, &ip->mask_len,
                              SSH_FORMAT_DATA,
                                ip->addr_data, SSH_IP_ADDR_LEN(ip),
                              SSH_FORMAT_END)) != (4 + SSH_IP_ADDR_LEN(ip)))
      return 0;

  point += got;

  /* Sanity check */
  if (!SSH_IP_IS4(ip) && !SSH_IP_IS6(ip))
    return 0;

  return point;
}

#if !(defined(_KERNEL) || defined(KERNEL))
size_t ssh_encode_ipaddr_buffer (SshBuffer buffer, const SshIpAddr ip)
{
  size_t got;
  unsigned char tmpbuf[1 + 4 + 16];

  if ((got = ssh_encode_ipaddr_array(tmpbuf, sizeof(tmpbuf), ip)) == 0)
      return FALSE;

  ssh_buffer_append(buffer, tmpbuf, got);

  return TRUE;
}

size_t ssh_ipaddr_decode_buffer (SshBuffer buffer, SshIpAddr ip)
{
  size_t got;

  if ((got = ssh_decode_ipaddr_array(ssh_buffer_ptr(buffer),
                                     ssh_buffer_len(buffer),
                                     ip)) == 0)
    return 0;

  ssh_buffer_consume(buffer, got);

  return got;
}
#endif
